@@ -7,6 +7,7 @@
 
 #include "db/db_test_util.h"
 #include "port/stack_trace.h"
+#include "table/unique_id_impl.h"
 
 namespace ROCKSDB_NAMESPACE {
 
@@ -288,16 +289,6 @@ TEST_F(CompactionServiceTest, BasicCompactions) {
         auto s = static_cast<Status*>(status);
         *s = Status::Aborted("MyTestCompactionService failed to compact!");
       });
-
-  // tracking success unique id verification
-  std::atomic_int verify_passed{0};
-  SyncPoint::GetInstance()->SetCallBack(
-      "Version::VerifySstUniqueIds::Passed", [&](void* arg) {
-        // override job status
-        auto id = static_cast<std::string*>(arg);
-        assert(!id->empty());
-        verify_passed++;
-      });
   SyncPoint::GetInstance()->EnableProcessing();
 
   Status s;
@@ -323,11 +314,18 @@ TEST_F(CompactionServiceTest, BasicCompactions) {
   }
   ASSERT_TRUE(s.IsAborted());
 
-  // Test verification
-  ASSERT_EQ(verify_passed, 0);
-  options.verify_sst_unique_id_in_manifest = true;
+  // Test re-open and successful unique id verification
+  std::atomic_int verify_passed{0};
+  SyncPoint::GetInstance()->SetCallBack(
+      "BlockBasedTable::Open::PassedVerifyUniqueId", [&](void* arg) {
+        // override job status
+        auto id = static_cast<UniqueId64x2*>(arg);
+        assert(*id != kNullUniqueId64x2);
+        verify_passed++;
+      });
   Reopen(options);
   ASSERT_GT(verify_passed, 0);
+  Close();
 }
 
 TEST_F(CompactionServiceTest, ManualCompaction) {
