@@ -28,7 +28,7 @@ namespace log {
 /**
  * Writer is a general purpose log stream writer. It provides an append-only
  * abstraction for writing data. The details of the how the data is written is
- * handled by the WriteableFile sub-class implementation.
+ * handled by the WritableFile sub-class implementation.
  *
  * File format:
  *
@@ -96,7 +96,7 @@ class Writer {
 
   IOStatus Close();
 
-  bool TEST_BufferIsEmpty();
+  bool BufferIsEmpty();
 
  private:
   std::unique_ptr<WritableFileWriter> dest_;
