@@ -79,6 +79,7 @@ void VersionEdit::Clear() {
   has_max_column_family_ = false;
   has_min_log_number_to_keep_ = false;
   has_last_sequence_ = false;
+  compact_cursors_.clear();
   deleted_files_.clear();
   new_files_.clear();
   blob_file_additions_.clear();
@@ -121,6 +122,13 @@ bool VersionEdit::EncodeTo(std::string* dst) const {
   if (has_last_sequence_) {
     PutVarint32Varint64(dst, kLastSequence, last_sequence_);
   }
+  for (size_t i = 0; i < compact_cursors_.size(); i++) {
+    if (compact_cursors_[i].second.Valid()) {
+      PutVarint32(dst, kCompactCursor);
+      PutVarint32(dst, compact_cursors_[i].first);  // level
+      PutLengthPrefixedSlice(dst, compact_cursors_[i].second.Encode());
+    }
+  }
   for (const auto& deleted : deleted_files_) {
     PutVarint32Varint32Varint64(dst, kDeletedFile, deleted.first /* level */,
                                 deleted.second /* file number */);
@@ -184,16 +192,6 @@ bool VersionEdit::EncodeTo(std::string* dst) const {
     PutVarint32(dst, NewFileCustomTag::kFileChecksumFuncName);
     PutLengthPrefixedSlice(dst, Slice(f.file_checksum_func_name));
 
-    if (f.max_timestamp != kDisableUserTimestamp) {
-      if (f.min_timestamp.size() != f.max_timestamp.size()) {
-        assert(false);
-        return false;
-      }
-      PutVarint32(dst, NewFileCustomTag::kMinTimestamp);
-      PutLengthPrefixedSlice(dst, Slice(f.min_timestamp));
-      PutVarint32(dst, NewFileCustomTag::kMaxTimestamp);
-      PutLengthPrefixedSlice(dst, Slice(f.max_timestamp));
-    }
     if (f.fd.GetPathId() != 0) {
       PutVarint32(dst, NewFileCustomTag::kPathId);
       char p = static_cast<char>(f.fd.GetPathId());
@@ -331,10 +329,6 @@ const char* VersionEdit::DecodeNewFile4From(Slice* input) {
         return "new-file4 custom field";
       }
       if (custom_tag == kTerminate) {
-        if (f.min_timestamp.size() != f.max_timestamp.size()) {
-          assert(false);
-          return "new-file4 custom field timestamp size mismatch error";
-        }
         break;
       }
       if (!GetLengthPrefixedSlice(input, &field)) {
@@ -395,12 +389,6 @@ const char* VersionEdit::DecodeNewFile4From(Slice* input) {
             }
           }
           break;
-        case kMinTimestamp:
-          f.min_timestamp = field.ToString();
-          break;
-        case kMaxTimestamp:
-          f.max_timestamp = field.ToString();
-          break;
         case kUniqueId:
           if (!DecodeUniqueIdBytes(field.ToString(), &f.unique_id).ok()) {
             f.unique_id = kNullUniqueId64x2;
@@ -512,15 +500,15 @@ Status VersionEdit::DecodeFrom(const Slice& src) {
         }
         break;
 
-      case kCompactPointer:
+      case kCompactCursor:
         if (GetLevel(&input, &level, &msg) &&
             GetInternalKey(&input, &key)) {
-          // we don't use compact pointers anymore,
-          // but we should not fail if they are still
-          // in manifest
+          // Here we re-use the output format of compact pointer in LevelDB
+          // to persist compact_cursors_
+          compact_cursors_.push_back(std::make_pair(level, key));
         } else {
           if (!msg) {
-            msg = "compaction pointer";
+            msg = "compaction cursor";
           }
         }
         break;
@@ -791,6 +779,12 @@ std::string VersionEdit::DebugString(bool hex_key) const {
     r.append("\n  LastSeq: ");
     AppendNumberTo(&r, last_sequence_);
   }
+  for (const auto& level_and_compact_cursor : compact_cursors_) {
+    r.append("\n  CompactCursor: ");
+    AppendNumberTo(&r, level_and_compact_cursor.first);
+    r.append(" ");
+    r.append(level_and_compact_cursor.second.DebugString(hex_key));
+  }
   for (const auto& deleted_file : deleted_files_) {
     r.append("\n  DeleteFile: ");
     AppendNumberTo(&r, deleted_file.first);
@@ -813,13 +807,6 @@ std::string VersionEdit::DebugString(bool hex_key) const {
       r.append(" blob_file:");
       AppendNumberTo(&r, f.oldest_blob_file_number);
     }
-    if (f.min_timestamp != kDisableUserTimestamp) {
-      assert(f.max_timestamp != kDisableUserTimestamp);
-      r.append(" min_timestamp:");
-      r.append(Slice(f.min_timestamp).ToString(true));
-      r.append(" max_timestamp:");
-      r.append(Slice(f.max_timestamp).ToString(true));
-    }
     r.append(" oldest_ancester_time:");
     AppendNumberTo(&r, f.oldest_ancester_time);
     r.append(" file_creation_time:");
@@ -838,6 +825,9 @@ std::string VersionEdit::DebugString(bool hex_key) const {
       r.append(" unique_id(internal): ");
       UniqueId64x2 id = f.unique_id;
       r.append(InternalUniqueIdToHumanString(&id));
+      r.append(" public_unique_id: ");
+      InternalUniqueIdToExternal(&id);
+      r.append(UniqueIdToHumanString(EncodeUniqueIdBytes(&id)));
     }
   }
 
@@ -938,11 +928,6 @@ std::string VersionEdit::DebugJSON(int edit_num, bool hex_key) const {
       jw << "FileSize" << f.fd.GetFileSize();
       jw << "SmallestIKey" << f.smallest.DebugString(hex_key);
       jw << "LargestIKey" << f.largest.DebugString(hex_key);
-      if (f.min_timestamp != kDisableUserTimestamp) {
-        assert(f.max_timestamp != kDisableUserTimestamp);
-        jw << "MinTimestamp" << Slice(f.min_timestamp).ToString(true);
-        jw << "MaxTimestamp" << Slice(f.max_timestamp).ToString(true);
-      }
       jw << "OldestAncesterTime" << f.oldest_ancester_time;
       jw << "FileCreationTime" << f.file_creation_time;
       jw << "FileChecksum" << Slice(f.file_checksum).ToString(true);
