@@ -4,23 +4,27 @@
 //  (found in the LICENSE.Apache file in the root directory).
 //
 // This file implements the "bridge" between Java and C++ and enables
-// calling c++ rocksdb::Statistics methods from Java side.
+// calling c++ ROCKSDB_NAMESPACE::Statistics methods from Java side.
+
+#include "rocksdb/statistics.h"
 
 #include <jni.h>
+
 #include <memory>
 #include <set>
 
 #include "include/org_rocksdb_Statistics.h"
+#include "rocksjni/cplusplus_to_java_convert.h"
 #include "rocksjni/portal.h"
 #include "rocksjni/statisticsjni.h"
-#include "rocksdb/statistics.h"
 
 /*
  * Class:     org_rocksdb_Statistics
  * Method:    newStatistics
  * Signature: ()J
  */
-jlong Java_org_rocksdb_Statistics_newStatistics__(JNIEnv* env, jclass jcls) {
+jlong Java_org_rocksdb_Statistics_newStatistics__(
+    JNIEnv* env, jclass jcls) {
   return Java_org_rocksdb_Statistics_newStatistics___3BJ(
       env, jcls, nullptr, 0);
 }
@@ -53,13 +57,12 @@ jlong Java_org_rocksdb_Statistics_newStatistics___3B(
  * Signature: ([BJ)J
  */
 jlong Java_org_rocksdb_Statistics_newStatistics___3BJ(
-    JNIEnv* env, jclass jcls, jbyteArray jhistograms,
-    jlong jother_statistics_handle) {
-
-  std::shared_ptr<rocksdb::Statistics>* pSptr_other_statistics = nullptr;
+    JNIEnv* env, jclass, jbyteArray jhistograms, jlong jother_statistics_handle) {
+  std::shared_ptr<ROCKSDB_NAMESPACE::Statistics>* pSptr_other_statistics =
+      nullptr;
   if (jother_statistics_handle > 0) {
     pSptr_other_statistics =
-        reinterpret_cast<std::shared_ptr<rocksdb::Statistics>*>(
+        reinterpret_cast<std::shared_ptr<ROCKSDB_NAMESPACE::Statistics>*>(
             jother_statistics_handle);
   }
 
@@ -68,14 +71,14 @@ jlong Java_org_rocksdb_Statistics_newStatistics___3BJ(
     const jsize len = env->GetArrayLength(jhistograms);
     if (len > 0) {
       jbyte* jhistogram = env->GetByteArrayElements(jhistograms, nullptr);
-      if (jhistogram == nullptr ) {
+      if (jhistogram == nullptr) {
         // exception thrown: OutOfMemoryError
         return 0;
       }
 
       for (jsize i = 0; i < len; i++) {
-        const rocksdb::Histograms histogram =
-            rocksdb::HistogramTypeJni::toCppHistograms(jhistogram[i]);
+        const ROCKSDB_NAMESPACE::Histograms histogram =
+            ROCKSDB_NAMESPACE::HistogramTypeJni::toCppHistograms(jhistogram[i]);
         histograms.emplace(histogram);
       }
 
@@ -83,15 +86,18 @@ jlong Java_org_rocksdb_Statistics_newStatistics___3BJ(
     }
   }
 
-  std::shared_ptr<rocksdb::Statistics> sptr_other_statistics = nullptr;
+  std::shared_ptr<ROCKSDB_NAMESPACE::Statistics> sptr_other_statistics =
+      nullptr;
   if (pSptr_other_statistics != nullptr) {
-      sptr_other_statistics =   *pSptr_other_statistics;
+    sptr_other_statistics = *pSptr_other_statistics;
   }
 
-  auto* pSptr_statistics = new std::shared_ptr<rocksdb::StatisticsJni>(
-      new rocksdb::StatisticsJni(sptr_other_statistics, histograms));
+  auto* pSptr_statistics =
+      new std::shared_ptr<ROCKSDB_NAMESPACE::StatisticsJni>(
+          new ROCKSDB_NAMESPACE::StatisticsJni(sptr_other_statistics,
+                                               histograms));
 
-  return reinterpret_cast<jlong>(pSptr_statistics);
+  return GET_CPLUSPLUS_POINTER(pSptr_statistics);
 }
 
 /*
@@ -100,10 +106,11 @@ jlong Java_org_rocksdb_Statistics_newStatistics___3BJ(
  * Signature: (J)V
  */
 void Java_org_rocksdb_Statistics_disposeInternal(
-    JNIEnv* env, jobject jobj, jlong jhandle) {
-  if(jhandle > 0) {
+    JNIEnv*, jobject, jlong jhandle) {
+  if (jhandle > 0) {
     auto* pSptr_statistics =
-        reinterpret_cast<std::shared_ptr<rocksdb::Statistics>*>(jhandle);
+        reinterpret_cast<std::shared_ptr<ROCKSDB_NAMESPACE::Statistics>*>(
+            jhandle);
     delete pSptr_statistics;
   }
 }
@@ -114,11 +121,13 @@ void Java_org_rocksdb_Statistics_disposeInternal(
  * Signature: (J)B
  */
 jbyte Java_org_rocksdb_Statistics_statsLevel(
-    JNIEnv* env, jobject jobj, jlong jhandle) {
+    JNIEnv*, jobject, jlong jhandle) {
   auto* pSptr_statistics =
-      reinterpret_cast<std::shared_ptr<rocksdb::Statistics>*>(jhandle);
+      reinterpret_cast<std::shared_ptr<ROCKSDB_NAMESPACE::Statistics>*>(
+          jhandle);
   assert(pSptr_statistics != nullptr);
-  return rocksdb::StatsLevelJni::toJavaStatsLevel(pSptr_statistics->get()->stats_level_);
+  return ROCKSDB_NAMESPACE::StatsLevelJni::toJavaStatsLevel(
+      pSptr_statistics->get()->get_stats_level());
 }
 
 /*
@@ -127,12 +136,14 @@ jbyte Java_org_rocksdb_Statistics_statsLevel(
  * Signature: (JB)V
  */
 void Java_org_rocksdb_Statistics_setStatsLevel(
-    JNIEnv* env, jobject jobj, jlong jhandle, jbyte jstats_level) {
+    JNIEnv*, jobject, jlong jhandle, jbyte jstats_level) {
   auto* pSptr_statistics =
-      reinterpret_cast<std::shared_ptr<rocksdb::Statistics>*>(jhandle);
+      reinterpret_cast<std::shared_ptr<ROCKSDB_NAMESPACE::Statistics>*>(
+          jhandle);
   assert(pSptr_statistics != nullptr);
-  auto stats_level = rocksdb::StatsLevelJni::toCppStatsLevel(jstats_level);
-  pSptr_statistics->get()->stats_level_ = stats_level;
+  auto stats_level =
+      ROCKSDB_NAMESPACE::StatsLevelJni::toCppStatsLevel(jstats_level);
+  pSptr_statistics->get()->set_stats_level(stats_level);
 }
 
 /*
@@ -141,12 +152,14 @@ void Java_org_rocksdb_Statistics_setStatsLevel(
  * Signature: (JB)J
  */
 jlong Java_org_rocksdb_Statistics_getTickerCount(
-    JNIEnv* env, jobject jobj, jlong jhandle, jbyte jticker_type) {
+    JNIEnv*, jobject, jlong jhandle, jbyte jticker_type) {
   auto* pSptr_statistics =
-      reinterpret_cast<std::shared_ptr<rocksdb::Statistics>*>(jhandle);
+      reinterpret_cast<std::shared_ptr<ROCKSDB_NAMESPACE::Statistics>*>(
+          jhandle);
   assert(pSptr_statistics != nullptr);
-  auto ticker = rocksdb::TickerTypeJni::toCppTickers(jticker_type);
-  return pSptr_statistics->get()->getTickerCount(ticker);
+  auto ticker = ROCKSDB_NAMESPACE::TickerTypeJni::toCppTickers(jticker_type);
+  uint64_t count = pSptr_statistics->get()->getTickerCount(ticker);
+  return static_cast<jlong>(count);
 }
 
 /*
@@ -155,11 +168,12 @@ jlong Java_org_rocksdb_Statistics_getTickerCount(
  * Signature: (JB)J
  */
 jlong Java_org_rocksdb_Statistics_getAndResetTickerCount(
-    JNIEnv* env, jobject jobj, jlong jhandle, jbyte jticker_type) {
+    JNIEnv*, jobject, jlong jhandle, jbyte jticker_type) {
   auto* pSptr_statistics =
-      reinterpret_cast<std::shared_ptr<rocksdb::Statistics>*>(jhandle);
+      reinterpret_cast<std::shared_ptr<ROCKSDB_NAMESPACE::Statistics>*>(
+          jhandle);
   assert(pSptr_statistics != nullptr);
-  auto ticker = rocksdb::TickerTypeJni::toCppTickers(jticker_type);
+  auto ticker = ROCKSDB_NAMESPACE::TickerTypeJni::toCppTickers(jticker_type);
   return pSptr_statistics->get()->getAndResetTickerCount(ticker);
 }
 
@@ -169,33 +183,38 @@ jlong Java_org_rocksdb_Statistics_getAndResetTickerCount(
  * Signature: (JB)Lorg/rocksdb/HistogramData;
  */
 jobject Java_org_rocksdb_Statistics_getHistogramData(
-    JNIEnv* env, jobject jobj, jlong jhandle, jbyte jhistogram_type) {
+    JNIEnv* env, jobject, jlong jhandle, jbyte jhistogram_type) {
   auto* pSptr_statistics =
-      reinterpret_cast<std::shared_ptr<rocksdb::Statistics>*>(jhandle);
+      reinterpret_cast<std::shared_ptr<ROCKSDB_NAMESPACE::Statistics>*>(
+          jhandle);
   assert(pSptr_statistics != nullptr);
 
-  rocksdb::HistogramData data;  // TODO(AR) perhaps better to construct a Java Object Wrapper that uses ptr to C++ `new HistogramData`
-  auto histogram = rocksdb::HistogramTypeJni::toCppHistograms(jhistogram_type);
-  pSptr_statistics->get()->histogramData(
-      static_cast<rocksdb::Histograms>(histogram), &data);
+  // TODO(AR) perhaps better to construct a Java Object Wrapper that
+  //    uses ptr to C++ `new HistogramData`
+  ROCKSDB_NAMESPACE::HistogramData data;
 
-  jclass jclazz = rocksdb::HistogramDataJni::getJClass(env);
-  if(jclazz == nullptr) {
+  auto histogram =
+      ROCKSDB_NAMESPACE::HistogramTypeJni::toCppHistograms(jhistogram_type);
+  pSptr_statistics->get()->histogramData(
+      static_cast<ROCKSDB_NAMESPACE::Histograms>(histogram), &data);
+
+  jclass jclazz = ROCKSDB_NAMESPACE::HistogramDataJni::getJClass(env);
+  if (jclazz == nullptr) {
     // exception occurred accessing class
     return nullptr;
   }
 
-  jmethodID mid = rocksdb::HistogramDataJni::getConstructorMethodId(
-      env);
-  if(mid == nullptr) {
+  jmethodID mid =
+      ROCKSDB_NAMESPACE::HistogramDataJni::getConstructorMethodId(env);
+  if (mid == nullptr) {
     // exception occurred accessing method
     return nullptr;
   }
 
-  return env->NewObject(
-      jclazz,
-      mid, data.median, data.percentile95,data.percentile99, data.average,
-      data.standard_deviation);
+  return env->NewObject(jclazz, mid, data.median, data.percentile95,
+                        data.percentile99, data.average,
+                        data.standard_deviation, data.max, data.count,
+                        data.sum, data.min);
 }
 
 /*
@@ -204,11 +223,13 @@ jobject Java_org_rocksdb_Statistics_getHistogramData(
  * Signature: (JB)Ljava/lang/String;
  */
 jstring Java_org_rocksdb_Statistics_getHistogramString(
-    JNIEnv* env, jobject jobj, jlong jhandle, jbyte jhistogram_type) {
+    JNIEnv* env, jobject, jlong jhandle, jbyte jhistogram_type) {
   auto* pSptr_statistics =
-      reinterpret_cast<std::shared_ptr<rocksdb::Statistics>*>(jhandle);
+      reinterpret_cast<std::shared_ptr<ROCKSDB_NAMESPACE::Statistics>*>(
+          jhandle);
   assert(pSptr_statistics != nullptr);
-  auto histogram = rocksdb::HistogramTypeJni::toCppHistograms(jhistogram_type);
+  auto histogram =
+      ROCKSDB_NAMESPACE::HistogramTypeJni::toCppHistograms(jhistogram_type);
   auto str = pSptr_statistics->get()->getHistogramString(histogram);
   return env->NewStringUTF(str.c_str());
 }
@@ -219,13 +240,14 @@ jstring Java_org_rocksdb_Statistics_getHistogramString(
  * Signature: (J)V
  */
 void Java_org_rocksdb_Statistics_reset(
-    JNIEnv* env, jobject jobj, jlong jhandle) {
-   auto* pSptr_statistics =
-      reinterpret_cast<std::shared_ptr<rocksdb::Statistics>*>(jhandle);
+    JNIEnv* env, jobject, jlong jhandle) {
+  auto* pSptr_statistics =
+      reinterpret_cast<std::shared_ptr<ROCKSDB_NAMESPACE::Statistics>*>(
+          jhandle);
   assert(pSptr_statistics != nullptr);
-  rocksdb::Status s = pSptr_statistics->get()->Reset();
+  ROCKSDB_NAMESPACE::Status s = pSptr_statistics->get()->Reset();
   if (!s.ok()) {
-   rocksdb::RocksDBExceptionJni::ThrowNew(env, s);
+    ROCKSDB_NAMESPACE::RocksDBExceptionJni::ThrowNew(env, s);
   }
 }
 
@@ -235,9 +257,10 @@ void Java_org_rocksdb_Statistics_reset(
  * Signature: (J)Ljava/lang/String;
  */
 jstring Java_org_rocksdb_Statistics_toString(
-    JNIEnv* env, jobject jobj, jlong jhandle) {
-   auto* pSptr_statistics =
-      reinterpret_cast<std::shared_ptr<rocksdb::Statistics>*>(jhandle);
+    JNIEnv* env, jobject, jlong jhandle) {
+  auto* pSptr_statistics =
+      reinterpret_cast<std::shared_ptr<ROCKSDB_NAMESPACE::Statistics>*>(
+          jhandle);
   assert(pSptr_statistics != nullptr);
   auto str = pSptr_statistics->get()->ToString();
   return env->NewStringUTF(str.c_str());
