@@ -37,20 +37,40 @@ PerfContext::PerfContext(const PerfContext& other) {
   block_read_byte = other.block_read_byte;
   block_read_time = other.block_read_time;
   block_cache_index_hit_count = other.block_cache_index_hit_count;
+  block_cache_standalone_handle_count =
+      other.block_cache_standalone_handle_count;
+  block_cache_real_handle_count = other.block_cache_real_handle_count;
   index_block_read_count = other.index_block_read_count;
   block_cache_filter_hit_count = other.block_cache_filter_hit_count;
   filter_block_read_count = other.filter_block_read_count;
   compression_dict_block_read_count = other.compression_dict_block_read_count;
   secondary_cache_hit_count = other.secondary_cache_hit_count;
+  compressed_sec_cache_insert_real_count =
+      other.compressed_sec_cache_insert_real_count;
+  compressed_sec_cache_insert_dummy_count =
+      other.compressed_sec_cache_insert_dummy_count;
+  compressed_sec_cache_uncompressed_bytes =
+      other.compressed_sec_cache_uncompressed_bytes;
+  compressed_sec_cache_compressed_bytes =
+      other.compressed_sec_cache_compressed_bytes;
   block_checksum_time = other.block_checksum_time;
   block_decompress_time = other.block_decompress_time;
   get_read_bytes = other.get_read_bytes;
   multiget_read_bytes = other.multiget_read_bytes;
   iter_read_bytes = other.iter_read_bytes;
+
+  blob_cache_hit_count = other.blob_cache_hit_count;
+  blob_read_count = other.blob_read_count;
+  blob_read_byte = other.blob_read_byte;
+  blob_read_time = other.blob_read_time;
+  blob_checksum_time = other.blob_checksum_time;
+  blob_decompress_time = other.blob_decompress_time;
+
   internal_key_skipped_count = other.internal_key_skipped_count;
   internal_delete_skipped_count = other.internal_delete_skipped_count;
   internal_recent_skipped_count = other.internal_recent_skipped_count;
   internal_merge_count = other.internal_merge_count;
+  internal_range_del_reseek_count = other.internal_range_del_reseek_count;
   write_wal_time = other.write_wal_time;
   get_snapshot_time = other.get_snapshot_time;
   get_from_memtable_time = other.get_from_memtable_time;
@@ -136,20 +156,40 @@ PerfContext::PerfContext(PerfContext&& other) noexcept {
   block_read_byte = other.block_read_byte;
   block_read_time = other.block_read_time;
   block_cache_index_hit_count = other.block_cache_index_hit_count;
+  block_cache_standalone_handle_count =
+      other.block_cache_standalone_handle_count;
+  block_cache_real_handle_count = other.block_cache_real_handle_count;
   index_block_read_count = other.index_block_read_count;
   block_cache_filter_hit_count = other.block_cache_filter_hit_count;
   filter_block_read_count = other.filter_block_read_count;
   compression_dict_block_read_count = other.compression_dict_block_read_count;
   secondary_cache_hit_count = other.secondary_cache_hit_count;
+  compressed_sec_cache_insert_real_count =
+      other.compressed_sec_cache_insert_real_count;
+  compressed_sec_cache_insert_dummy_count =
+      other.compressed_sec_cache_insert_dummy_count;
+  compressed_sec_cache_uncompressed_bytes =
+      other.compressed_sec_cache_uncompressed_bytes;
+  compressed_sec_cache_compressed_bytes =
+      other.compressed_sec_cache_compressed_bytes;
   block_checksum_time = other.block_checksum_time;
   block_decompress_time = other.block_decompress_time;
   get_read_bytes = other.get_read_bytes;
   multiget_read_bytes = other.multiget_read_bytes;
   iter_read_bytes = other.iter_read_bytes;
+
+  blob_cache_hit_count = other.blob_cache_hit_count;
+  blob_read_count = other.blob_read_count;
+  blob_read_byte = other.blob_read_byte;
+  blob_read_time = other.blob_read_time;
+  blob_checksum_time = other.blob_checksum_time;
+  blob_decompress_time = other.blob_decompress_time;
+
   internal_key_skipped_count = other.internal_key_skipped_count;
   internal_delete_skipped_count = other.internal_delete_skipped_count;
   internal_recent_skipped_count = other.internal_recent_skipped_count;
   internal_merge_count = other.internal_merge_count;
+  internal_range_del_reseek_count = other.internal_range_del_reseek_count;
   write_wal_time = other.write_wal_time;
   get_snapshot_time = other.get_snapshot_time;
   get_from_memtable_time = other.get_from_memtable_time;
@@ -237,20 +277,40 @@ PerfContext& PerfContext::operator=(const PerfContext& other) {
   block_read_byte = other.block_read_byte;
   block_read_time = other.block_read_time;
   block_cache_index_hit_count = other.block_cache_index_hit_count;
+  block_cache_standalone_handle_count =
+      other.block_cache_standalone_handle_count;
+  block_cache_real_handle_count = other.block_cache_real_handle_count;
   index_block_read_count = other.index_block_read_count;
   block_cache_filter_hit_count = other.block_cache_filter_hit_count;
   filter_block_read_count = other.filter_block_read_count;
   compression_dict_block_read_count = other.compression_dict_block_read_count;
   secondary_cache_hit_count = other.secondary_cache_hit_count;
+  compressed_sec_cache_insert_real_count =
+      other.compressed_sec_cache_insert_real_count;
+  compressed_sec_cache_insert_dummy_count =
+      other.compressed_sec_cache_insert_dummy_count;
+  compressed_sec_cache_uncompressed_bytes =
+      other.compressed_sec_cache_uncompressed_bytes;
+  compressed_sec_cache_compressed_bytes =
+      other.compressed_sec_cache_compressed_bytes;
   block_checksum_time = other.block_checksum_time;
   block_decompress_time = other.block_decompress_time;
   get_read_bytes = other.get_read_bytes;
   multiget_read_bytes = other.multiget_read_bytes;
   iter_read_bytes = other.iter_read_bytes;
+
+  blob_cache_hit_count = other.blob_cache_hit_count;
+  blob_read_count = other.blob_read_count;
+  blob_read_byte = other.blob_read_byte;
+  blob_read_time = other.blob_read_time;
+  blob_checksum_time = other.blob_checksum_time;
+  blob_decompress_time = other.blob_decompress_time;
+
   internal_key_skipped_count = other.internal_key_skipped_count;
   internal_delete_skipped_count = other.internal_delete_skipped_count;
   internal_recent_skipped_count = other.internal_recent_skipped_count;
   internal_merge_count = other.internal_merge_count;
+  internal_range_del_reseek_count = other.internal_range_del_reseek_count;
   write_wal_time = other.write_wal_time;
   get_snapshot_time = other.get_snapshot_time;
   get_from_memtable_time = other.get_from_memtable_time;
@@ -335,20 +395,35 @@ void PerfContext::Reset() {
   block_read_byte = 0;
   block_read_time = 0;
   block_cache_index_hit_count = 0;
+  block_cache_standalone_handle_count = 0;
+  block_cache_real_handle_count = 0;
   index_block_read_count = 0;
   block_cache_filter_hit_count = 0;
   filter_block_read_count = 0;
   compression_dict_block_read_count = 0;
   secondary_cache_hit_count = 0;
+  compressed_sec_cache_insert_real_count = 0;
+  compressed_sec_cache_insert_dummy_count = 0;
+  compressed_sec_cache_uncompressed_bytes = 0;
+  compressed_sec_cache_compressed_bytes = 0;
   block_checksum_time = 0;
   block_decompress_time = 0;
   get_read_bytes = 0;
   multiget_read_bytes = 0;
   iter_read_bytes = 0;
+
+  blob_cache_hit_count = 0;
+  blob_read_count = 0;
+  blob_read_byte = 0;
+  blob_read_time = 0;
+  blob_checksum_time = 0;
+  blob_decompress_time = 0;
+
   internal_key_skipped_count = 0;
   internal_delete_skipped_count = 0;
   internal_recent_skipped_count = 0;
   internal_merge_count = 0;
+  internal_range_del_reseek_count = 0;
   write_wal_time = 0;
 
   get_snapshot_time = 0;
@@ -457,20 +532,33 @@ std::string PerfContext::ToString(bool exclude_zero_counters) const {
   PERF_CONTEXT_OUTPUT(block_read_byte);
   PERF_CONTEXT_OUTPUT(block_read_time);
   PERF_CONTEXT_OUTPUT(block_cache_index_hit_count);
+  PERF_CONTEXT_OUTPUT(block_cache_standalone_handle_count);
+  PERF_CONTEXT_OUTPUT(block_cache_real_handle_count);
   PERF_CONTEXT_OUTPUT(index_block_read_count);
   PERF_CONTEXT_OUTPUT(block_cache_filter_hit_count);
   PERF_CONTEXT_OUTPUT(filter_block_read_count);
   PERF_CONTEXT_OUTPUT(compression_dict_block_read_count);
   PERF_CONTEXT_OUTPUT(secondary_cache_hit_count);
+  PERF_CONTEXT_OUTPUT(compressed_sec_cache_insert_real_count);
+  PERF_CONTEXT_OUTPUT(compressed_sec_cache_insert_dummy_count);
+  PERF_CONTEXT_OUTPUT(compressed_sec_cache_uncompressed_bytes);
+  PERF_CONTEXT_OUTPUT(compressed_sec_cache_compressed_bytes);
   PERF_CONTEXT_OUTPUT(block_checksum_time);
   PERF_CONTEXT_OUTPUT(block_decompress_time);
   PERF_CONTEXT_OUTPUT(get_read_bytes);
   PERF_CONTEXT_OUTPUT(multiget_read_bytes);
   PERF_CONTEXT_OUTPUT(iter_read_bytes);
+  PERF_CONTEXT_OUTPUT(blob_cache_hit_count);
+  PERF_CONTEXT_OUTPUT(blob_read_count);
+  PERF_CONTEXT_OUTPUT(blob_read_byte);
+  PERF_CONTEXT_OUTPUT(blob_read_time);
+  PERF_CONTEXT_OUTPUT(blob_checksum_time);
+  PERF_CONTEXT_OUTPUT(blob_decompress_time);
   PERF_CONTEXT_OUTPUT(internal_key_skipped_count);
   PERF_CONTEXT_OUTPUT(internal_delete_skipped_count);
   PERF_CONTEXT_OUTPUT(internal_recent_skipped_count);
   PERF_CONTEXT_OUTPUT(internal_merge_count);
+  PERF_CONTEXT_OUTPUT(internal_range_del_reseek_count);
   PERF_CONTEXT_OUTPUT(write_wal_time);
   PERF_CONTEXT_OUTPUT(get_snapshot_time);
   PERF_CONTEXT_OUTPUT(get_from_memtable_time);
