@@ -10,11 +10,6 @@
 // A filter block is stored near the end of a Table file.  It contains
 // filters (e.g., bloom filters) for all data blocks in the table combined
 // into a single filter block.
-//
-// It is a base class for BlockBasedFilter and FullFilter.
-// These two are both used in BlockBasedTable. The first one contain filter
-// For a part of keys in sst file, the second contain filter for all keys
-// in sst file.
 
 #pragma once
 
@@ -44,12 +39,10 @@ using MultiGetRange = MultiGetContext::Range;
 
 // A FilterBlockBuilder is used to construct all of the filters for a
 // particular Table.  It generates a single string which is stored as
-// a special block in the Table.
+// a special block in the Table, or partitioned into smaller filters.
 //
 // The sequence of calls to FilterBlockBuilder must match the regexp:
-//      (StartBlock Add*)* Finish
-//
-// BlockBased/Full FilterBlock would be called in the same way.
+//      Add* Finish
 class FilterBlockBuilder {
  public:
   explicit FilterBlockBuilder() {}
@@ -59,8 +52,6 @@ class FilterBlockBuilder {
 
   virtual ~FilterBlockBuilder() {}
 
-  virtual bool IsBlockBased() = 0;                    // If is blockbased filter
-  virtual void StartBlock(uint64_t block_offset) = 0;  // Start new block filter
   virtual void Add(
       const Slice& key_without_ts) = 0;        // Add a key to current filter
   virtual bool IsEmpty() const = 0;            // Empty == none added
@@ -108,8 +99,6 @@ class FilterBlockReader {
   FilterBlockReader(const FilterBlockReader&) = delete;
   FilterBlockReader& operator=(const FilterBlockReader&) = delete;
 
-  virtual bool IsBlockBased() = 0;  // If is blockbased filter
-
   /**
    * If no_io is set, then it returns true if it cannot answer the query without
    * reading data from disk. This is used in PartitionedFilterBlockReader to
@@ -120,23 +109,21 @@ class FilterBlockReader {
    * built upon InternalKey and must be provided via const_ikey_ptr when running
    * queries.
    */
-  virtual bool KeyMayMatch(const Slice& key,
-                           const SliceTransform* prefix_extractor,
-                           uint64_t block_offset, const bool no_io,
+  virtual bool KeyMayMatch(const Slice& key, const bool no_io,
                            const Slice* const const_ikey_ptr,
                            GetContext* get_context,
-                           BlockCacheLookupContext* lookup_context) = 0;
+                           BlockCacheLookupContext* lookup_context,
+                           Env::IOPriority rate_limiter_priority) = 0;
 
-  virtual void KeysMayMatch(MultiGetRange* range,
-                            const SliceTransform* prefix_extractor,
-                            uint64_t block_offset, const bool no_io,
-                            BlockCacheLookupContext* lookup_context) {
+  virtual void KeysMayMatch(MultiGetRange* range, const bool no_io,
+                            BlockCacheLookupContext* lookup_context,
+                            Env::IOPriority rate_limiter_priority) {
     for (auto iter = range->begin(); iter != range->end(); ++iter) {
       const Slice ukey_without_ts = iter->ukey_without_ts;
       const Slice ikey = iter->ikey;
       GetContext* const get_context = iter->get_context;
-      if (!KeyMayMatch(ukey_without_ts, prefix_extractor, block_offset, no_io,
-                       &ikey, get_context, lookup_context)) {
+      if (!KeyMayMatch(ukey_without_ts, no_io, &ikey, get_context,
+                       lookup_context, rate_limiter_priority)) {
         range->SkipKey(iter);
       }
     }
@@ -145,25 +132,25 @@ class FilterBlockReader {
   /**
    * no_io and const_ikey_ptr here means the same as in KeyMayMatch
    */
-  virtual bool PrefixMayMatch(const Slice& prefix,
-                              const SliceTransform* prefix_extractor,
-                              uint64_t block_offset, const bool no_io,
+  virtual bool PrefixMayMatch(const Slice& prefix, const bool no_io,
                               const Slice* const const_ikey_ptr,
                               GetContext* get_context,
-                              BlockCacheLookupContext* lookup_context) = 0;
+                              BlockCacheLookupContext* lookup_context,
+                              Env::IOPriority rate_limiter_priority) = 0;
 
   virtual void PrefixesMayMatch(MultiGetRange* range,
                                 const SliceTransform* prefix_extractor,
-                                uint64_t block_offset, const bool no_io,
-                                BlockCacheLookupContext* lookup_context) {
+                                const bool no_io,
+                                BlockCacheLookupContext* lookup_context,
+                                Env::IOPriority rate_limiter_priority) {
     for (auto iter = range->begin(); iter != range->end(); ++iter) {
       const Slice ukey_without_ts = iter->ukey_without_ts;
       const Slice ikey = iter->ikey;
       GetContext* const get_context = iter->get_context;
       if (prefix_extractor->InDomain(ukey_without_ts) &&
-          !PrefixMayMatch(prefix_extractor->Transform(ukey_without_ts),
-                          prefix_extractor, block_offset, no_io, &ikey,
-                          get_context, lookup_context)) {
+          !PrefixMayMatch(prefix_extractor->Transform(ukey_without_ts), no_io,
+                          &ikey, get_context, lookup_context,
+                          rate_limiter_priority)) {
         range->SkipKey(iter);
       }
     }
@@ -188,7 +175,8 @@ class FilterBlockReader {
                              const Slice* const const_ikey_ptr,
                              bool* filter_checked, bool need_upper_bound_check,
                              bool no_io,
-                             BlockCacheLookupContext* lookup_context) = 0;
+                             BlockCacheLookupContext* lookup_context,
+                             Env::IOPriority rate_limiter_priority) = 0;
 };
 
 }  // namespace ROCKSDB_NAMESPACE
